/*
 * vanadiel-clock.js
 *
 * Copyright (c) 2004 by Daniel Potter.  All rights reserved.
 *
 * This file is part of the Vana'diel Clock.
 *
 * The Vana'diel Clock is free software; you can redistribute
 * it and/or modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * The Vana'diel Clock is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the Vana'diel Clock; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 *
 * Implements a Vana'diel clock in the browser window.
 *
 * Simple, minimally useful.
 */

// Name of Vana'diel days - note that these days are the value we give to the
// "day" attribute on our time marker to set the day icon.

const vanaDays = [ "fire", "earth", "water", "wind", "ice", "thunder", "light", "dark" ];

var vanaLocalNames = new Array();

// Name of moon phases, used to look up the phase name and set the moon icon
const moonPhases = [ "new", "waxingcrescent", "firstquarter", "waxinggibbous", "full", "waninggibbous", "lastquarter", "waningcrescent" ];

var moonPhaseLocalNames = new Array();
var moonPhaseLocalWaxing;
var moonPhaseLocalWaning;

var vanadielOffset = 0;

{
    // Load our localized strings
    var strBundleService = Components.classes["@mozilla.org/intl/stringbundle;1"].getService(Components.interfaces.nsIStringBundleService);
    var strings = strBundleService.createBundle("chrome://vanadielclock/locale/clock.properties");

    for (var i in vanaDays) {
	vanaLocalNames[i] = strings.GetStringFromName("day." + vanaDays[i]);
    }
    for (var i in moonPhases) {
	moonPhaseLocalNames[i] = strings.GetStringFromName("moon." + moonPhases[i]);
    }
    moonPhaseLocalWaxing = strings.GetStringFromName("moon.waxing");
    moonPhaseLocalWaning = strings.GetStringFromName("moon.waning");
    // Calculate the offset to convert to Vana'diel time for the current time zone

    // I, unfortunately, have no idea who originally came up with the following.
    // Note that it has since been converted to GMT.
    
    // Earth time:       2002/06/23 15:00:00 GMT
    // Vana'diel time:   0898/02/01 00:00:00 (VST?)

    var vanaTime  = (898 * 360 + 30) * 24 * 60 * 60 * 1000 / 25;
    var baseDay = new Date();
    baseDay.setUTCFullYear(2002, 5, 23); // Set date to 2003-06-23
    baseDay.setUTCHours(15, 0, 0, 0);    // Set time to 15:00:00.0000

    vanadielOffset = baseDay.getTime() - vanaTime;

    // check to see if the clock is even active
    var useClock = true;
    var useMoon = true;
    var useDate = false;
    
    var prefs = Components.classes["@mozilla.org/preferences-service;1"].
               getService(Components.interfaces.nsIPrefBranch);
    try {
	useClock = prefs.getBoolPref("vanadiel.clock.show");
    } catch (ex) {
	// guess so
    }
    try {
	useMoon = prefs.getBoolPref("vanadiel.clock.moon");
    } catch (ex) {
	// guess so
    }
    try {
	useDate = prefs.getBoolPref("vanadiel.clock.date");
    } catch (ex) {
	// guess not
    }

    if (useClock) {
	// add an onload listener
	var handler;
	eval("handler = function(event) { startVanadielClock(" + useMoon + ", " + useDate + ") }");
	//dump(handler + "\n");
	addEventListener("load",
		handler,
		false);
    }
}

var vanadielClockPanel = null;
var vanadielMoonIcon = null;
var vanadielDayIcon = null;
var vanadielDateLabel = null;
var vanadielTimeLabel = null;
var vanadielClockInterval = null;
var vanadielLastDay = -1;

function startVanadielClock(showMoon, showDate) {
    // show the moon on null
    if (showMoon == null) {
	showMoon = true;
    }
    // don't show the date on null (kinda redundant, actually)
    if (showDate == null) {
	showDate = false;
    }
    if (vanadielClockInterval == null) {
	vanadielClockPanel = document.getElementById("vanadiel-clock");
	vanadielClockPanel.style.display = "-moz-box";
	vanadielMoonIcon = document.getElementById("vanadiel-moon");
	vanadielMoonIcon.style.display = showMoon ? "-moz-box" : "none";
	vanadielDayIcon = document.getElementById("vanadiel-day");
	vanadielDateLabel = document.getElementById("vanadiel-date");
	vanadielDateLabel.style.display = showDate ? "-moz-box" : "none";
	vanadielTimeLabel = document.getElementById("vanadiel-time");
	// immediately update the clock
	vanadielLastDay = -1;
	updateVanadielClock();
	// calculate the timeout to the next second boundary
	var timeout = (new Date().getTime() + vanadielOffset) % 2400;
	// Starting at that timeout, update the clock every Vana'diel minute,
	// which is 60000/25 = 2400 milliseconds
	vanadielClockInterval = setTimeout('vanadielClockInterval = setInterval("updateVanadielClock()")', timeout);
    } else {
	// this may have just toggled the moon/date
	vanadielMoonIcon = document.getElementById("vanadiel-moon");
	vanadielMoonIcon.style.display = showMoon ? "-moz-box" : "none";
	vanadielDateLabel = document.getElementById("vanadiel-date");
	vanadielDateLabel.style.display = showDate ? "-moz-box" : "none";
    }
}

function stopVanadielClock() {
    if (vanadielClockInterval != null) {
	vanadielClockPanel = document.getElementById("vanadiel-clock");
	vanadielClockPanel.style.display = "none";
	clearInterval(vanadielClockInterval);
	vanadielClockInterval = null;
    }
}

function updateVanadielClock() {
    // Vana'diel time runs 25 times faster than our time - this is the time
    // in "Vana'diel milliseconds" actually.
    var vanadielTime = (new Date().getTime() - vanadielOffset) * 25;
    // We only display the day, hour, and minute in the little clock
    var vanadielDay = Math.floor(vanadielTime / 86400000);
    // Only update day related stuff on - well, a new day
    if (vanadielDay != vanadielLastDay) {
	vanadielLastDay = vanadielDay;
	var vanadielYear = Math.floor(vanadielDay / 360);
	var vanadielMonth = Math.floor((vanadielDay % 360) / 30) + 1;
	var vanadielDayMonth = (vanadielDay % 30) + 1;
	var vanadielDayWeek = vanadielDay % 8;
	// calculate the current moon phase - unlike the current
	// day of the week, this isn't nice and happily rooted at 0
	// moonphase = 0 - full moon
	// moonphase = 42 - new moon
	var moonphase = (vanadielDay + 26) % 84;
	// translate that value into a percentage value
	var moonpercent = ((42 - moonphase) * 100) / 42;
	// negative values are "waxing" and backwards
	var waxing = false;
	if (moonpercent < 0) {
	    waxing = true;
	    moonpercent = Math.abs(moonpercent);
	}
	// Now make it an interger
	moonpercent = Math.floor(moonpercent + 0.5);
	// Translate the phase into one of the eight phases.
	//
	// To see how the "phase counter" stacks up to a percent, run the
	// following Perl 1-liner:
	//
	// perl -e 'for($p=0;$p<84;$p++){print "$p = ".int((abs((42-$p)*100/42)+0.5))."\% (wa".($p<42?"n":"x")."ing)\n";}'
	//
	// Simple, huh?
	//
	// === MOON PHASES ===
	// 80- 2 == 90%(Wax)  - 95%(Wane) - Full
	//  3-16 == 93%(Wane) - 62%(Wane) - Waning Gibbous
	// 17-23 == 60%(Wane) - 45%(Wane) - Last Quarter
	// 24-37 == 43%(Wane) - 12%(Wane) - Waning Crescent
	// 38-44 == 10%(Wane) -  5%(Wax)  - New Moon
	// 45-58 ==  7%(Wax)  - 38%(Wax)  - Waxing Crescent
	// 59-65 == 40%(Wax)  - 55%(Wax)  - First Quarter
	// 66-79 == 57%(Wax)  - 88%(Wax)  - Waxing Gibbous
	var basicMoonPhase = 4; // full
	if (moonphase > 3 && moonphase < 17) {
	    basicMoonPhase = 5; // waning gibbous
	} else if (moonphase < 24) {
	    basicMoonPhase = 6; // last quarter
	} else if (moonphase < 38) {
	    basicMoonPhase = 7; // waning crescent
	} else if (moonphase < 45) {
	    basicMoonPhase = 0; // new
	} else if (moonphase < 59) {
	    basicMoonPhase = 1; // waxing crescent
	} else if (moonphase < 66) {
	    basicMoonPhase = 2; // first quarter
	} else if (moonphase < 80) {
	    basicMoonPhase = 3; // waxing gibbous
	}
	var date = vanaLocalNames[vanadielDayWeek] + ", " + vanadielDayMonth + " / ";
	date += vanadielMonth + " / ";
	date += vanadielYear;
	//dump("Calculated moon at " + moonpercent + "% " + (waxing ? "waxing" : "waning"));
	//dump("\nPhase counter is " + moonphase + ", basic phase is " + basicMoonPhase + " (" + moonPhases[basicMoonPhase] + ")\n");
	vanadielMoonIcon.setAttribute("phase", moonPhases[basicMoonPhase]);
	var moonPhaseLabel = moonpercent + "% " + moonPhaseLocalNames[basicMoonPhase];
	if (basicMoonPhase == 0 || basicMoonPhase == 4) {
	    // add "waxing" or "waning" to this if you can't tell
	    // (ie, when the moon is new or full, so you know if it's
	    // moving to/from the "true" full/new state)
	    if (moonphase != 0 && moonphase != 42) {
		// But don't bother showing waxing/waning when the moon
		// really is completely full or completely new - it should
		// (hopefully) be obvious that it's going to move in the
		// other direction.
		moonPhaseLabel += " (" + (waxing ? moonPhaseLocalWaxing : moonPhaseLocalWaning) + ")";
	    }
	}
	vanadielMoonIcon.setAttribute("tooltiptext", moonPhaseLabel);
	vanadielClockPanel.setAttribute("tooltiptext", date);
	vanadielDayIcon.setAttribute("tooltiptext", date);
	vanadielDayIcon.setAttribute("day", vanaDays[vanadielDayWeek]);
	vanadielDateLabel.setAttribute("value", date);
	vanadielTimeLabel.setAttribute("tooltiptext", date);
    }
    var vanadielHour = Math.floor(vanadielTime / 3600000) % 24;
    var vanadielMinute = Math.floor(vanadielTime / 60000) % 60;
    var label = vanadielHour + ":";
    if (vanadielMinute < 10) {
	label += "0";
    }
    label += vanadielMinute;
    vanadielTimeLabel.setAttribute("value", label);
}

function VanadielClock_showOptions() {
    var win = window.openDialog("chrome://vanadielclock/content/options.xul", "optwin", "chrome");
    if (win) {
	win.focus();
    }
}

function vanadielClockClick(event) {
    if (event.detail == 2) {
	VanadielClock_showOptions();
    }
}
